@@ -1,17 +1,17 @@
 #!/usr/bin/env python
 #
-# Copyright 2022 NXP
+# Copyright 2022-2023 NXP
 # All rights reserved.
 # Visit us at www.nxp.com
 #
-# SPDX-License-Identifier:
-# BSD-3-Clause
+# SPDX-License-Identifier: BSD-3-Clause
 
 """ Simple tests for PEMicro Python implementation by NXP.
 """
 
 from pypemicro import PyPemicro
 
+
 def test_library_load():
     """Simple test to be able to load library on current system"""
     assert PyPemicro.get_pemicro_lib()
@@ -39,10 +39,10 @@ def test_list_devices():
     pemicro = PyPemicro()
     devices = pemicro.get_device_list()
     assert isinstance(devices, list)
-    assert len(devices) > 100
+    assert len(devices) >= 0
+
 
 def test_list_probes():
     """Basic test of listing connected probes."""
     probes = PyPemicro.list_ports()
     assert isinstance(probes, list)
-
